//
//    Derived from Dire Wolf by John Langner, WB2OSZ (to whom I send many compliments)
//    and modified by me, IK1PLD: www.fotografAle.it
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; See the GNU General Public License for more details.
//
//    pkt2wave.c is a program for generating AFSK AX.25 frames to audio Wav
//               it is developed to be used together with nbfm (by IK1PLD)
//               on the Raspberry Pi, running Debian.
//
//  To compile: gcc -DUSE_ALSA -O3 -pthread -o pkt2wave pkt2wave.c ax25_pad.c hdlc_send.c fcs_calc.c gen_tone.c textcolor.c -lm
//
//  Please refer to the original code for further information, remarks and for the required libraries.
//


#include <stdio.h>     
#include <stdlib.h>    
#include <getopt.h>
#include <string.h>
#include <assert.h>

#include "audio.h"
#include "ax25_pad.h"
#include "hdlc_send.h"
#include "gen_tone.h"
#include "textcolor.h"

#define KNRM  "\x1B[0m"  //normal
#define KCYN  "\x1B[36m" //cyan
#define BOLDYELLOW "\033[1m\033[33m"


static void usage (char **argv);
static int audio_file_open (char *fname, struct audio_s *pa);
static int audio_file_close (void);



int main(int argc, char **argv)
{
    int c;
    int digit_optind = 0;
    int err;
    unsigned char fbuf[AX25_MAX_PACKET_LEN+2];
    int flen;
    int packet_count = 0;
    int i;
    int chan;
	char TxString [400]="IK1PLD-1>BEACON,IR1SVA-2:This is the pkt2wave + nbfm by ik1pld for Raspberry";
	char *TxStringPointer=TxString;


// Set up default values for the modem.
        struct audio_s modem;

        modem.num_channels = DEFAULT_NUM_CHANNELS;              /* mono */
        modem.samples_per_sec = DEFAULT_SAMPLES_PER_SEC;        /* -r option */
        modem.bits_per_sample = DEFAULT_BITS_PER_SAMPLE;        /* 16 bits */
        
	for (chan = 0; chan < MAX_CHANS; chan++) {
	  modem.modem_type[chan] = AFSK;				            /* AFSK or SCRAMBLE */
	  modem.mark_freq[chan] = DEFAULT_MARK_FREQ;                /* 1200 Hz*/
          modem.space_freq[chan] = DEFAULT_SPACE_FREQ;          /* 2200 Hz*/
          modem.baud[chan] = DEFAULT_BAUD;                      /* 1200 bps */
	}

// Set up other default values.
    int amplitude = 50;			/* -a option */
    int leading_zeros = 12;		/* -z option */
    char output_file[256];		/* -o option */
    FILE *input_fp = NULL;		/* File or NULL for built-in message */

    packet_t pp;


    strcpy (output_file, "");


    while (1) {
        int this_option_optind = optind ? optind : 1;
        int option_index = 0;
        static struct option long_options[] = {
            {"future1", 1, 0, 0},
            {"future2", 0, 0, 0},
            {"future3", 1, 0, 'c'},
            {0, 0, 0, 0}
        };

	/* ':' following option character means arg is required. */

        c = getopt_long(argc, argv, "a:B:r:n:o:z:",
                        long_options, &option_index);
        if (c == -1)
            break;

        switch (c) {

          case 'B':		/* -B for data Bit rate */
						/*    300 implies 1600/1800 AFSK. */
						/*    1200 implies 1200/2200 AFSK. */
						/*    9600 implies G3RUH scrambled. */
            modem.baud[0] = atoi(optarg);
            dw_printf ("Data rate set to %d bits / second.\n", modem.baud[0]);
            if (modem.baud[0] < 100 || modem.baud[0] > 10000) {
              dw_printf (BOLDYELLOW "Use a more reasonable bit rate in range of 100 - 10000.\n");
              exit (EXIT_FAILURE);
            }
				switch (modem.baud[0]) {
					case 300:
						modem.mark_freq[0] = 1600;
						modem.space_freq[0] = 1800;
						break;
					case 1200:
						modem.mark_freq[0] = 1200;
						modem.space_freq[0] = 2200;
						break;
					case 9600:
						modem.modem_type[0] = SCRAMBLE;
						dw_printf ("Using scrambled baseband signal rather than AFSK.\n");
					break;
				}
            break;

          case 'n':				/* -n number of packets (repeated consequently) */
			packet_count = atoi(optarg);
            break;

          case 'a':				/* -a for amplitude */
            amplitude = atoi(optarg);
            dw_printf ("Amplitude set to %d%%.\n", amplitude);
            if (amplitude < 0 || amplitude > 100) {
				dw_printf (BOLDYELLOW "Amplitude must be in range of 0 to 100.\n");
				exit (EXIT_FAILURE);
				}
            break;

          case 'r':				/* -r for audio sample Rate */
            modem.samples_per_sec = atoi(optarg);
            dw_printf ("Audio sample rate set to %d samples / second.\n", modem.samples_per_sec);
            if (modem.samples_per_sec < MIN_SAMPLES_PER_SEC || modem.samples_per_sec > MAX_SAMPLES_PER_SEC) {
				dw_printf (BOLDYELLOW "Use a more reasonable audio sample rate in range of %d - %d.\n",
						MIN_SAMPLES_PER_SEC, MAX_SAMPLES_PER_SEC);
              exit (EXIT_FAILURE);
				}
            break;

          case 'z':				/* -z leading zeros before frame flag */
            leading_zeros = atoi(optarg);
            dw_printf ("Send %d zero bits before frame flag.\n", leading_zeros);
			/* The demodulator needs a few for the clock recovery PLL. */
			/* We don't want to be here all day either. */
            /* We can't translast to time yet because the data bit rate */
            /* could be changed later. */

            if (leading_zeros < 8 || leading_zeros > 12000) {
				dw_printf (BOLDYELLOW "Use a more reasonable value.\n");
              exit (EXIT_FAILURE);
				}
            break;

          case 'o':				/* -o for Output file */
            strcpy (output_file, optarg);
            dw_printf ("Output file set to %s\n", output_file);
            break;

          case '?':
            /* Unknown option message was already printed. */
            usage (argv);
            break;

          default:
            /* Should not be here. */
            dw_printf("?? getopt returned character code 0%o ??\n", c);
            usage (argv);
        }
    }

    if (optind < argc) {
		char str[400];

        if (optind < argc - 1) {
			dw_printf ("Warning: File(s) beyond the first are ignored.\n");
			}	

        if (strcmp(argv[optind], "-") == 0) {
			dw_printf ("Reading from stdin ...\n");
			input_fp = stdin;
			}
        else {
          input_fp = fopen(argv[optind], "r");
          if (input_fp == NULL) {
 	    dw_printf (BOLDYELLOW "Can't open %s for read.\n", argv[optind]);
            exit (EXIT_FAILURE);
          }
          dw_printf ("Reading from %s ...\n", argv[optind]);    
        }

        while (fgets (str, sizeof(str), input_fp) != NULL) {
			}

        if (input_fp != stdin) {
			fclose (input_fp);
			}
		strcpy (TxString, str);
		}
    else {
		dw_printf ("built in message...\n");
		}
   

        if (strlen(output_file) == 0) {
          dw_printf (BOLDYELLOW "ERROR: The -o ouput file option must be specified.\n");
          usage (argv);
          exit (1);
        }

	err = audio_file_open (output_file, &modem);


        if (err < 0) {
          dw_printf (BOLDYELLOW "ERROR - Can't open output file.\n");
          exit (1);
        }


	gen_tone_init (&modem, amplitude);

        assert (modem.bits_per_sample == 8 || modem.bits_per_sample == 16);
        assert (modem.num_channels == 1 || modem.num_channels == 2);
        assert (modem.samples_per_sec >= MIN_SAMPLES_PER_SEC && modem.samples_per_sec <= MAX_SAMPLES_PER_SEC);


// Generate packets (repeated)
	  for (i = 1; i <= packet_count; i++) {
	    char stemp[272];
		dw_printf ("%s\n", TxString);
	    sprintf (stemp, TxStringPointer);
	    pp = ax25_from_text (stemp, 1);
	    flen = ax25_pack (pp, fbuf);
	      hdlc_send_flags (0, 8, 0);
	      hdlc_send_frame (0, fbuf, flen);
	      hdlc_send_flags (0, 2, 1);
	    ax25_delete (pp);
	  }
	audio_file_close();
    	return EXIT_SUCCESS;
}


static void usage (char **argv)
{
	dw_printf (KCYN "\n");
	dw_printf ("Packet radio to Wav converter v20140902 by IK1PLD\n");
	dw_printf ("Usage: %s [options] [file]\n",argv[0]);
	dw_printf ("Options:\n");
	dw_printf ("  -a <number>   Signal amplitude in range of 0 - 100%%.  Default 50.\n");
	dw_printf ("  -B <number>   bits/second for data. Proper modem selected for 300, 1200, 9600.\n");
	dw_printf ("  -r <number>   Audio sample Rate.  Default is %d.\n", DEFAULT_SAMPLES_PER_SEC);
	dw_printf ("  -n <number>   Generate specified number of repeated frames.\n");
	dw_printf ("  -o <file>     Send output to .wav file (mandatory).\n");
	dw_printf ("  -z <number>   Number of leading zero bits before frame.\n");
	dw_printf ("                  Default is 12 which is .01 seconds at 1200 bps.\n");

	dw_printf ("\n");
	dw_printf ("An optional file may be specified to provide messages other than\n");
	dw_printf ("the default built-in message. The format should correspond to:\n");
	dw_printf ("CALL1>CALL2,CALL3,CALLn:80ch free text...\n");
	dw_printf ("Example: IK1PLD-1>APRS,IW1PPB-2:!4401.14NS08120.83E#\n");
	dw_printf ("\n");
	dw_printf ("Example:  %s -o ik1pld.wav\n", argv[0]);
       dw_printf ("  With all defaults, a built-in test message is generated\n");
       dw_printf ("  with standard Bell 202 tones used for packet radio on ordinary\n");
       dw_printf ("  VHF FM transceivers, and saved to ik1pld.wav\n");
	dw_printf ("\n");
	dw_printf ("Example: %s -B 9600\n", argv[0]);
        dw_printf ("  9600 bps G3RUH mode.\n");
	dw_printf ("\n");
	dw_printf ("Example: %s -B 300\n", argv[0]);
        dw_printf ("  200 Hz shift, 300 bps, suitable for HF SSB transceiver.\n");
	dw_printf ("\n");
	dw_printf ("Example:  echo -n \"IK1PLD>WORLD:Hello, world!\" | %s -a 25 -o x.wav -\n", argv[0]);
        dw_printf ("  Read message from stdin and put quarter volume sound into the file x.wav.\n\n" KNRM);

	exit (EXIT_FAILURE);
}



/*------------------------------------------------------------------
 * Name:        audio_file_open
 * Purpose:     Open a .WAV format file for output.
 * Inputs:      fname		- Name of .WAV file to create.
 *
 *		pa		- Address of structure of type audio_s.
 *				
 *				The fields that we care about are:
 *					num_channels
 *					samples_per_sec
 *					bits_per_sample
 *				If zero, reasonable defaults will be provided.
 *         
 * Returns:     0 for success, -1 for failure.
 *----------------------------------------------------------------*/

struct wav_header {             /* .WAV file header. */
        char riff[4];           /* "RIFF" */
        int filesize;          /* file length - 8 */
        char wave[4];           /* "WAVE" */
        char fmt[4];            /* "fmt " */
        int fmtsize;           /* 16. */
        short wformattag;       /* 1 for PCM. */
        short nchannels;        /* 1 for mono, 2 for stereo. */
        int nsamplespersec;    /* sampling freq, Hz. */
        int navgbytespersec;   /* = nblockalign * nsamplespersec. */
        short nblockalign;      /* = wbitspersample / 8 * nchannels. */
        short wbitspersample;   /* 16 or 8. */
        char data[4];           /* "data" */
        int datasize;          /* number of bytes following. */
} ;

				/* 8 bit samples are unsigned bytes */
				/* in range of 0 .. 255. */
 
				/* 16 bit samples are signed short */
				/* in range of -32768 .. +32767. */

static FILE *out_fp = NULL;

static struct wav_header header;

static int byte_count;			/* Number of data bytes written to file. */
					/* Will be written to header when file is closed. */


static int audio_file_open (char *fname, struct audio_s *pa)
{
	int n;

/*
 * Fill in defaults for any missing values.
 */
	if (pa -> num_channels == 0)
	  pa -> num_channels = DEFAULT_NUM_CHANNELS;

	if (pa -> samples_per_sec == 0)
	  pa -> samples_per_sec = DEFAULT_SAMPLES_PER_SEC;

	if (pa -> bits_per_sample == 0)
	  pa -> bits_per_sample = DEFAULT_BITS_PER_SAMPLE;


/*
 * Write the file header.  Don't know length yet.
 */
        out_fp = fopen (fname, "wb");	
	
        if (out_fp == NULL) {
		   dw_printf (BOLDYELLOW "Couldn't open file for write: %s\n", fname);
	   perror ("");
           return (-1);
        }

	memset (&header, 0, sizeof(header));

        memcpy (header.riff, "RIFF", (size_t)4);
        header.filesize = 0; 
        memcpy (header.wave, "WAVE", (size_t)4);
        memcpy (header.fmt, "fmt ", (size_t)4);
        header.fmtsize = 16;			// Always 16.
        header.wformattag = 1;     		// 1 for PCM.

        header.nchannels = pa -> num_channels;   		
        header.nsamplespersec = pa -> samples_per_sec;    
        header.wbitspersample = pa -> bits_per_sample;  
		
        header.nblockalign = header.wbitspersample / 8 * header.nchannels;     
        header.navgbytespersec = header.nblockalign * header.nsamplespersec;   
        memcpy (header.data, "data", (size_t)4);
        header.datasize = 0;        

	assert (header.nchannels == 1 || header.nchannels == 2);

        n = fwrite (&header, sizeof(header), (size_t)1, out_fp);

	if (n != 1) {
	  dw_printf (BOLDYELLOW "Couldn't write header to: %s\n", fname);
	  perror ("");
	  fclose (out_fp);
	  out_fp = NULL;
          return (-1);
        }


/*
 * Number of bytes written will be filled in later.
 */
        byte_count = 0;
	
	return (0);

} /* end audio_open */


/*------------------------------------------------------------------
 * Name:        audio_put
 * Purpose:     Send one byte to the audio output file.
 * Inputs:	c	- One byte in range of 0 - 255.
 * Returns:     Normally non-negative.
 *              -1 for any type of error.
 *
 * Description:	The caller must deal with the details of mono/stereo
 *		and number of bytes per sample.
 *----------------------------------------------------------------*/


int audio_put (int c)
{
	static short sample16;
	int s;
    byte_count++;
	return (putc(c, out_fp));
} /* end audio_put */


int audio_flush ()
{
	return 0;
}

/*------------------------------------------------------------------
 * Name:        audio_file_close
 * Purpose:     Close the audio output file.
 * Returns:     Normally non-negative.
 *              -1 for any type of error.
 *
 * Description:	Must go back to beginning of file and fill in the
 *		size of the data.
 *----------------------------------------------------------------*/

static int audio_file_close (void)
{
	int n;

/*
 * Go back and fix up lengths in header.
 */
        header.filesize = byte_count + sizeof(header) - 8;           
        header.datasize = byte_count;        

	if (out_fp == NULL) {
	  return (-1);
 	}

        fflush (out_fp);

        fseek (out_fp, 0L, SEEK_SET);         
        n = fwrite (&header, sizeof(header), (size_t)1, out_fp);

	if (n != 1) {
	  dw_printf (BOLDYELLOW "Couldn't write header to audio file.\n");
	  perror ("");		// TODO: remove perror.
	  fclose (out_fp);
	  out_fp = NULL;
          return (-1);
        }

        fclose (out_fp);
        out_fp = NULL;

	return (0);

} /* end audio_close */

